#include <stdio.h>
#include <stdlib.h>
#define E0 10000000 // size of memory blocks

/*
./convert graph.txt graph.bin
graph.txt is the edge list with one edge on each line, it can be foloweg by a weight.
exemple for a triangle graph:
0 1
1 2
2 3
exemple for a weighted triangle graph:
0 1 0.123
1 2 45.6
2 3 7
*/


typedef struct {
	unsigned dim;	//dimensions of the squared matrix
	unsigned *degcum;	//cumulative degree
	unsigned *neigh;	//list of neighbors
	double *weight;	//Weights
} sparse;

void sparse2file(sparse *mat,char* file_name) {
	unsigned n=mat->dim,e=mat->degcum[n];
	FILE *file;

	file=fopen(file_name,"w");
	if (mat->weight==NULL)
		printf("Writing unweighted graph to file %s\n",file_name);
	else
		printf("Writing weighted graph to file %s\n",file_name);

	fwrite(&n,sizeof(unsigned),1,file);
	fwrite(mat->degcum,sizeof(unsigned),n+1,file);
	fwrite(mat->neigh,sizeof(unsigned),e,file);

	if (mat->weight!=NULL)
		fwrite(mat->weight,sizeof(double),e,file);

	fclose(file);
	printf("Graph written\n");
}



//compute the maximum of three unsignedegers
unsigned max3(unsigned a,unsigned b,unsigned c){
	a=(a>b) ? a : b;
	return (a>c) ? a : c;
}



int main(int argc,char** argv){
	unsigned k=0, i, e=0, e1=E0, n=0;
	unsigned *a1=malloc(E0*sizeof(unsigned)), *a2=malloc(E0*sizeof(unsigned));
	unsigned *deg, *t;
	unsigned char c;
	double *weight0=0;
	sparse *mat=malloc(sizeof(sparse));
	FILE *file;

	file=fopen(argv[1],"r");
	c=getc(file);
	while (c!='\n') {
		if (c==' ') {
			k++;
		}
		c=getc(file);
	}
	
	if (k==1) {
		mat->weight=NULL;
		}
	else if (k==2) {
		weight0=malloc(E0*sizeof(double));
	}
	else {
		printf("error: input file format\n");
	}
	fclose(file);


	if (k==1) {
		printf("Reading unweighted graph from file %s\n",argv[1]);
		file=fopen(argv[1],"r");
		while (feof(file)==0) {
			fscanf(file,"%u %u", &a1[e], &a2[e]);
			n=max3(n,a1[e],a2[e]);
			if (e++==e1) {
				e1+=E0;
				a1=realloc(a1,e1*sizeof(unsigned));
				a2=realloc(a2,e1*sizeof(unsigned));
			}
		}
		printf("Converting unweighted graph\n");
	}
	else {
		printf("Reading weighted graph from file %s\n",argv[1]);
		file=fopen(argv[1],"r");
		while (feof(file)==0) {
			fscanf(file,"%u %u %lf", &a1[e], &a2[e], &weight0[e]);
			n=max3(n,a1[e],a2[e]);
			if (e++==e1) {
				e1+=E0;
				a1=realloc(a1,e1*sizeof(unsigned));
				a2=realloc(a2,e1*sizeof(unsigned));
				weight0=realloc(weight0,e1*sizeof(double));
			}
		}
		printf("Converting weighted graph\n");
	}

	fclose(file);
	n++,e--;
	mat->dim=n;

	deg=calloc(n,sizeof(unsigned));
	for (i=0;i<e;i++) {
		deg[a1[i]]++;
		deg[a2[i]]++;
	}

	mat->degcum=calloc(n+1,sizeof(unsigned));
	for (i=1;i<n+1;i++) {
		mat->degcum[i]=mat->degcum[i-1]+deg[i-1];
	}

	free(deg);

	t=calloc(n,sizeof(unsigned));

	mat->neigh=calloc(2*e,sizeof(unsigned));

	if (k==1) {
		for (i=0;i<e;i++) {
			mat->neigh[ mat->degcum[a1[i]] + t[ a1[i] ]++ ]=a2[i];
			mat->neigh[ mat->degcum[a2[i]] + t[ a2[i] ]++ ]=a1[i];
		}
	}
	else {
		mat->weight=calloc(2*e,sizeof(double));
		for (i=0;i<e;i++) {
			mat->neigh[ mat->degcum[a1[i]] + t[ a1[i] ] ]=a2[i];
			mat->neigh[ mat->degcum[a2[i]] + t[ a2[i] ] ]=a1[i];
			mat->weight[ mat->degcum[a1[i]] + t[ a1[i] ]++ ]=weight0[i];
			mat->weight[ mat->degcum[a2[i]] + t[ a2[i] ]++ ]=weight0[i];
		}
	}

	free(a1),free(a2),free(t);

	sparse2file(mat,argv[2]);

	return 0;
}



